/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.1.6
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2020, zouyaoji <370681295@qq.com>
 */
import _Object$assign from '@babel/runtime-corejs2/core-js/object/assign';
import { isFunction, isArray, isString, isObject, isEmptyObj } from './util';

/**
 * 将对象 {x: number, y: number } 转换为 Cesium.Cartesian2
 * @param {Object} val
 * @returns {Cartesian2 | CallbackProperty} 返回 Cartesian2 或者 CallbackProperty
 */

function makeCartesian2(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium = Cesium,
      Cartesian2 = _Cesium.Cartesian2,
      CallbackProperty = _Cesium.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  } else if (val && val.hasOwnProperty('x')) {
    return new Cartesian2(val.x, val.y);
  }

  return val;
}
/**
 * 将对象 { x: number, y: number, z: number } 或者 { lng: number, lat: number, height: number }转换为 Cesium.Cartesian3
 * @param {Object} val
 * @param {Boolean} isConstant 传入function时生效，true 代表回调 function 每时每刻都返回值， false 代表改变才会返回值。默认false。
 * @returns 返回 Cartesian3 或者 CallbackProperty
 */

function makeCartesian3(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium2 = Cesium,
      CallbackProperty = _Cesium2.CallbackProperty,
      Cartesian3 = _Cesium2.Cartesian3;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  } else if (val && val.hasOwnProperty('x')) {
    return new Cartesian3(val.x, val.y, val.z);
  } else if (val && val.hasOwnProperty('lng')) {
    return Cartesian3.fromDegrees(val.lng, val.lat, val.height);
  } else if (isArray(val)) {
    return new Cartesian3(val[0], val[1], val[2]);
  }

  return val;
}
/**
 * 将数组 [lng, lat, height, ……，lng, lat, height] 转换为 Cesium.Cartesian3 数组
 * @param {Array} val
 * @returns {Array<Cartesian3>}
 */

function makeCartesian3Array(vals) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium3 = Cesium,
      CallbackProperty = _Cesium3.CallbackProperty,
      Cartesian3 = _Cesium3.Cartesian3;

  if (isFunction(vals)) {
    return new CallbackProperty(vals, isConstant);
  } else if (vals && isArray(vals) && vals[0] instanceof Cartesian3 || vals._callback) {
    return vals;
  }

  var coordinates = [];
  vals.forEach(function (item) {
    coordinates.push(item.lng);
    coordinates.push(item.lat);
    coordinates.push(item.height);
  });
  return coordinates.length >= 3 ? Cartesian3.fromDegreesArrayHeights(coordinates) : vals;
}
/**
 * 将形如 [lng, lat, ……，lng, lat] 数组转换为 Cesium.Cartesian2 数组
 * @param {Array} vals
 * @returns {Array<Cartesian2>}
 */

function makeCartesian2Array(vals, isConstant) {
  var _Cesium4 = Cesium,
      CallbackProperty = _Cesium4.CallbackProperty,
      Cartesian2 = _Cesium4.Cartesian2;

  if (isFunction(vals)) {
    return new CallbackProperty(vals, isConstant);
  }

  var cartesian2Array = [];
  vals.forEach(function (item) {
    cartesian2Array.push(new Cartesian2(item.x, item.y));
  });
  return cartesian2Array;
}
/**
 * 将对象 {x: number, y: number, z: number ,w: number} 转换为 Cesium.Quaternion
 * @param {Object} val
 */

function makeQuaternion(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium5 = Cesium,
      CallbackProperty = _Cesium5.CallbackProperty,
      Quaternion = _Cesium5.Quaternion;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  }

  return val && val.hasOwnProperty('x') ? new Quaternion(val.x, val.y, val.z, val.w) : val;
}
/**
 * 解析 HierarchyJson
 * @param {Object} val
 */

function parsePolygonHierarchyJson(val) {
  val.forEach(function (element) {
    element.positions = makeCartesian3Array(element.positions);

    if (element.holes) {
      parsePolygonHierarchyJson(element.holes);
    }
  });
}
/**
 * 普通数组或对象转 Cesium.PolygonHierarchy 对象。
 * @param {Object|Array} val
 */


function makePolygonHierarchy(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium6 = Cesium,
      PolygonHierarchy = _Cesium6.PolygonHierarchy,
      CallbackProperty = _Cesium6.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  }

  if (isArray(val) && val.length >= 3) {
    return new PolygonHierarchy(makeCartesian3Array(val));
  }

  if (Cesium.defined(val.positions)) {
    val.positions = makeCartesian3Array(val.positions);
    parsePolygonHierarchyJson(val.holes);
  }

  return val;
}
/**
 * 普通对象 {near: number, nearValue: number, far: number, farValue: number} 转 Cesium.NearFarScalar 对象。
 * @param {Object} val
 * @returns {NearFarScalar}
 */

function makeNearFarScalar(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium7 = Cesium,
      NearFarScalar = _Cesium7.NearFarScalar,
      CallbackProperty = _Cesium7.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  }

  if (val && val.hasOwnProperty('near')) {
    return new NearFarScalar(val.near, val.nearValue, val.far, val.farValue);
  }

  return val;
}
/**
 * 普通对象 {near: number, far: number} 转 Cesium.DistanceDisplayCondition 对象。
 * @param {Object} val
 * @returns {DistanceDisplayCondition}
 */

function makeDistanceDisplayCondition(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium8 = Cesium,
      DistanceDisplayCondition = _Cesium8.DistanceDisplayCondition,
      CallbackProperty = _Cesium8.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  }

  if (val && val.hasOwnProperty('near')) {
    return new DistanceDisplayCondition(val.near, val.far);
  }

  return val;
}
/**
 * 普通对象或数组 [r, g, b, a] 或字符串转 Cesium.Color 对象。
 * @param {String|Array|Object} val
 * @returns {Color}
 */

function makeColor(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium9 = Cesium,
      Color = _Cesium9.Color,
      CallbackProperty = _Cesium9.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  } else if (val instanceof Color) {
    return val;
  } else if (isArray(val)) {
    return new Cesium.Color(val[0], val[1], val[2], val[3]);
  } else if (isString(val)) {
    return Color.fromCssColorString(val);
  }

  return val;
}
/**
 * 普通对象或数组 [r, g, b, a] 或字符串转 MaterialProperty
 * @param {String|Array|Object} val
 */

function makeMaterialProperty(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium10 = Cesium,
      CallbackProperty = _Cesium10.CallbackProperty,
      Color = _Cesium10.Color,
      Resource = _Cesium10.Resource,
      ColorMaterialProperty = _Cesium10.ColorMaterialProperty,
      ImageMaterialProperty = _Cesium10.ImageMaterialProperty,
      PolylineArrowMaterialProperty = _Cesium10.PolylineArrowMaterialProperty,
      PolylineDashMaterialProperty = _Cesium10.PolylineDashMaterialProperty,
      PolylineGlowMaterialProperty = _Cesium10.PolylineGlowMaterialProperty,
      PolylineOutlineMaterialProperty = _Cesium10.PolylineOutlineMaterialProperty,
      CheckerboardMaterialProperty = _Cesium10.CheckerboardMaterialProperty,
      GridMaterialProperty = _Cesium10.GridMaterialProperty,
      StripeMaterialProperty = _Cesium10.StripeMaterialProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  }

  if (/(.*)\.(jpg|bmp|gif|ico|pcx|jpeg|tif|png|raw|tga)$/.test(val) || val instanceof Resource || val instanceof HTMLCanvasElement || val instanceof HTMLVideoElement) {
    var result = new ImageMaterialProperty();
    result.image = val;
    return result;
  }

  if (isArray(val) || isString(val)) {
    return new ColorMaterialProperty(makeColor(val));
  }

  if (val && val.hasOwnProperty('fabric')) {
    switch (val.fabric.type) {
      case 'Image':
        return new ImageMaterialProperty({
          image: val.fabric.uniforms.image,
          repeat: val.fabric.uniforms.repeat || makeCartesian2({
            x: 1.0,
            y: 1.0
          }),
          color: makeColor(val.fabric.uniforms.color) || Color.WHITE,
          transparent: val.fabric.uniforms.transparent || false
        });

      case 'Color':
        return new ColorMaterialProperty(makeColor(val.fabric.uniforms.color || Color.WHITE));

      case 'PolylineArrow':
        return new PolylineArrowMaterialProperty(makeColor(val.fabric.uniforms.color || Color.WHITE));

      case 'PolylineDash':
        return new PolylineDashMaterialProperty({
          color: makeColor(val.fabric.uniforms.color) || Color.WHITE,
          gapColor: val.fabric.uniforms.gapColor || Color.TRANSPARENT,
          dashLength: val.fabric.uniforms.taperPower || 16.0,
          dashPattern: val.fabric.uniforms.taperPower || 255.0
        });

      case 'PolylineGlow':
        return new PolylineGlowMaterialProperty({
          color: makeColor(val.fabric.uniforms.color) || Color.WHITE,
          glowPower: val.fabric.uniforms.glowPower || 0.25,
          taperPower: val.fabric.uniforms.taperPower || 1.0
        });

      case 'PolylineOutline':
        return new PolylineOutlineMaterialProperty({
          color: makeColor(val.fabric.uniforms.color) || Color.WHITE,
          outlineColor: val.fabric.uniforms.outlineColor || Color.BLACK,
          outlineWidth: val.fabric.uniforms.outlineWidth || 1.0
        });

      case 'Checkerboard':
        return new CheckerboardMaterialProperty({
          evenColor: makeColor(val.fabric.uniforms.evenColor) || Color.WHITE,
          oddColor: val.fabric.uniforms.oddColor || Color.BLACK,
          repeat: val.fabric.uniforms.repeat || makeCartesian2({
            x: 2,
            y: 2
          })
        });

      case 'Grid':
        return new GridMaterialProperty({
          color: makeColor(val.fabric.uniforms.color) || Color.WHITE,
          cellAlpha: val.fabric.uniforms.cellAlpha || 0.1,
          lineCount: val.fabric.uniforms.lineCount || makeCartesian2({
            x: 8,
            y: 8
          }),
          lineThickness: val.fabric.uniforms.lineThickness || makeCartesian2({
            x: 1,
            y: 1
          }),
          lineOffset: val.fabric.uniforms.lineOffset || makeCartesian2({
            x: 0,
            y: 0
          })
        });

      case 'Stripe':
        return new StripeMaterialProperty({
          orientation: makeColor(val.fabric.uniforms.orientation) || 0,
          evenColor: val.fabric.uniforms.evenColor || Color.WHITE,
          oddColor: val.fabric.uniforms.oddColor || Color.BLACK,
          offset: val.fabric.uniforms.offset || 0,
          repeat: val.fabric.uniforms.repeat || 1
        });
    }
  }

  return val;
}
/**
 * 转 Material
 * @param {String|Array|Object} val
 */

function makeMaterial(val) {
  var cmpName = this.$options.name;

  if (cmpName && cmpName.indexOf('graphics') !== -1) {
    return makeMaterialProperty(val);
  }

  if (val && val.hasOwnProperty('fabric')) {
    var f = function f(obj) {
      for (var i in obj) {
        if (!isArray(obj[i]) && isObject(obj[i])) {
          f(obj[i]);
        } else {
          if (i.toLocaleLowerCase().indexOf('color') !== -1 && !isEmptyObj(obj[i])) {
            var result = makeColor(obj[i]); // Cesium 通过对象属性个数判断具体材质类型的，通过 Cesium.combine 移除 vue 传的一些属性

            obj[i] = Cesium.combine(result, result, true);
          }
        }
      }
    };

    f(val);
    return new Cesium.Material(val);
  }

  return val;
}
/**
 * 将对象 {west: number, south: number, east: number, north: number} 或者[west, south, east, north]数组 转 Cesium.Rectangle 对象。
 * @param {Object} val
 * @returns {Rectangle}
 */

function makeRectangle(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium11 = Cesium,
      Rectangle = _Cesium11.Rectangle,
      RectangleGraphics = _Cesium11.RectangleGraphics,
      CallbackProperty = _Cesium11.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  } // Entiy 的 rectangle 属性不能调用这个方法


  if (val instanceof RectangleGraphics) {
    return val;
  }

  if (isArray(val) && val.length === 4) {
    return Rectangle.fromDegrees(val[0], val[1], val[2], val[3]);
  }

  if (val && val.hasOwnProperty('west')) {
    return Rectangle.fromDegrees(val.west, val.south, val.east, val.north);
  }

  return val;
}
/**
 * 将对象 {x: number, y: number, width: number, height: number} 转 Cesium.BoundingRectangle 对象。
 * @param {Object} val
 * @returns {BoundingRectangle}
 */

function makeBoundingRectangle(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium12 = Cesium,
      BoundingRectangle = _Cesium12.BoundingRectangle,
      CallbackProperty = _Cesium12.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  }

  if (val && val.hasOwnProperty('x')) {
    return new BoundingRectangle(val.x, val.y, val.width, val.height);
  }

  return val;
}
/**
 * 普通对象 {normal: number, distance: number} 转 Cesium.Plane 对象。
 * @param {Object} val
 * @returns {Plane}
 */

function makePlane(val) {
  var isConstant = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var _Cesium13 = Cesium,
      Cartesian3 = _Cesium13.Cartesian3,
      Plane = _Cesium13.Plane,
      PlaneGraphics = _Cesium13.PlaneGraphics,
      CallbackProperty = _Cesium13.CallbackProperty;

  if (isFunction(val)) {
    return new CallbackProperty(val, isConstant);
  } // Entiy 和 PlaneGraphics 都有个 plane 属性 要区别一下


  if (val instanceof PlaneGraphics) {
    return val;
  }

  if (val && val.hasOwnProperty('normal')) {
    Cartesian3.normalize(makeCartesian3(val.normal), val.normal);
    return new Plane(val.normal, val.distance);
  }

  return val;
}
/**
 * 普通对象转平移、旋转、缩放变换对象。
 * @param {*} val
 */

function makeTranslationRotationScale(val) {
  if (val && val.hasOwnProperty('translation')) {
    return new Cesium.TranslationRotationScale(makeCartesian3(val.translation), makeQuaternion(val.rotation), makeCartesian3(val.scale));
  }

  return val;
}
function makeOptions(val) {
  var cmpName = this.$options.name;

  switch (cmpName) {
    case 'vc-datasource-geojson':
      var result = {};

      _Object$assign(result, val);

      result && result.markerColor && (result.markerColor = makeColor(result.markerColor));
      result && result.stroke && (result.stroke = makeColor(result.stroke));
      result && result.fill && (result.fill = makeColor(result.fill));
      return result;
  }

  return val;
}

export { makeBoundingRectangle, makeCartesian2, makeCartesian2Array, makeCartesian3, makeCartesian3Array, makeColor, makeDistanceDisplayCondition, makeMaterial, makeMaterialProperty, makeNearFarScalar, makeOptions, makePlane, makePolygonHierarchy, makeQuaternion, makeRectangle, makeTranslationRotationScale };
