class ChangeablePrimitive {
  initialiseOptions (options) {
    fillOptions(this, options)

    this._ellipsoid = undefined
    this._granularity = undefined
    this._height = undefined
    this._textureRotationAngle = undefined
    this._id = undefined

    // set the flags to initiate a first drawing
    this._createPrimitive = true
    this._primitive = undefined
    this._outlinePolygon = undefined
  }

  setAttribute (name, value) {
    this[name] = value
    this._createPrimitive = true
  }

  getAttribute (name) {
    return this[name]
  }

  update (context, frameState, commandList) {
    if (!Cesium.defined(this.ellipsoid)) {
      throw new Cesium.DeveloperError('this.ellipsoid must be defined.')
    }

    if (!Cesium.defined(this.appearance)) {
      throw new Cesium.DeveloperError('this.material must be defined.')
    }

    if (this.granularity < 0.0) {
      throw new Cesium.DeveloperError('this.granularity and scene2D/scene3D overrides must be greater than zero.')
    }

    if (!this.show) {
      return
    }

    if (!this._createPrimitive && (!Cesium.defined(this._primitive))) {
      // No positions/hierarchy to draw
      return
    }

    if (this._createPrimitive ||
      (this._ellipsoid !== this.ellipsoid) ||
      (this._granularity !== this.granularity) ||
      (this._height !== this.height) ||
      (this._textureRotationAngle !== this.textureRotationAngle) ||
      (this._id !== this.id)) {
      var geometry = this.getGeometry()
      if (!geometry) {
        return
      }

      this._createPrimitive = false
      this._ellipsoid = this.ellipsoid
      this._granularity = this.granularity
      this._height = this.height
      this._textureRotationAngle = this.textureRotationAngle
      this._id = this.id

      this._primitive = this._primitive && this._primitive.destroy()

      this._primitive = new Cesium.Primitive({
        geometryInstances: new Cesium.GeometryInstance({
          geometry: geometry,
          id: this.id,
          pickPrimitive: this
        }),
        appearance: this.appearance,
        asynchronous: this.asynchronous
      })

      this._outlinePolygon = this._outlinePolygon && this._outlinePolygon.destroy()
      if (this.strokeColor && this.getOutlineGeometry) {
        // create the highlighting frame
        this._outlinePolygon = new Cesium.Primitive({
          geometryInstances: new Cesium.GeometryInstance({
            geometry: this.getOutlineGeometry(),
            attributes: {
              color: Cesium.ColorGeometryInstanceAttribute.fromColor(this.strokeColor)
            }
          }),
          appearance: new Cesium.PerInstanceColorAppearance({
            flat: true,
            renderState: {
              depthTest: {
                enabled: true
              },
              lineWidth: 4.0
            }
          })
        })
      }
    }

    var primitive = this._primitive
    primitive.appearance.material = this.material
    primitive.debugShowBoundingVolume = this.debugShowBoundingVolume
    primitive.update(context, frameState, commandList)
    this._outlinePolygon && this._outlinePolygon.update(context, frameState, commandList)
  }

  isDestroyed () {
    return false
  }

  destroy () {
    this._primitive = this._primitive && this._primitive.destroy()
    return Cesium.destroyObject(this)
  }

  setStrokeStyle (strokeColor, strokeWidth) {
    if (!this.strokeColor || !this.strokeColor.equals(strokeColor) || this.strokeWidth !== strokeWidth) {
      this._createPrimitive = true
      this.strokeColor = strokeColor
      this.strokeWidth = strokeWidth
    }
  }
}

export default ChangeablePrimitive

function clone (from, to) {
  if (from == null || typeof from !== 'object') return from
  if (from.constructor !== Object && from.constructor !== Array) return from
  if (from.constructor === Date || from.constructor === RegExp || from.constructor === Function ||
    from.constructor === String || from.constructor === Number || from.constructor === Boolean) { return new from.constructor(from) }

  to = to || new from.constructor()

  for (var name in from) {
    to[name] = typeof to[name] === 'undefined' ? clone(from[name], null) : to[name]
  }

  return to
}

function fillOptions (options, defaultOptions) {
  options = options || {}
  var option
  for (option in defaultOptions) {
    if (options[option] === undefined) {
      options[option] = clone(defaultOptions[option])
    }
  }
}
